//
// Random maze generator
//
// By Vincent van Beveren
//

function Point(x, y) 
{
  this.x = x;
  this.y = y;
}

var Direction = {
	NORTH : "north",
	SOUTH : "south",
	WEST : "west",
	EAST : "east"
}

function Maze(width, height) 
{
  if (width % 2 == 0 || height % 2 == 0) 
  {
    throw Error("Can't create maze of even size");
  }
  this.width = width;
  this.height = height;
  this.reset = function() 
  {
    this.maze = new Array(this.height);
    for (y = 0; y < this.height; y++) 
    {
      this.maze[y] = new Array(this.width);
      for (x = 0; x < this.width; x++) 
      {
        this.maze[y][x] = 1;
      }
    }
  }
  this.set = function(x, y, v) 
  {
    this.maze[y][x] = v;
  }
  this.get = function(x, y) 
  {
    return this.maze[y][x];
  }
  this.generate = function() 
  {
    this.reset();
    stack = new Array();
    start = new Point(33,33);
    stack.push(start);
    this.set(start.x, start.y, 0);
    while (stack.length > 0) 
    {
      l = stack.pop();
      dir = this.getSomeDirection(l);
      if (dir != null) 
      {
        stack.push(l);
        if (dir == Direction.SOUTH) 
        {
          stack.push(new Point(l.x, l.y + 2));
          this.set(l.x, l.y + 1, 0);
          this.set(l.x, l.y + 2, 0);
        }
        else if (dir == Direction.NORTH) 
        {
          stack.push(new Point(l.x, l.y - 2));
          this.set(l.x, l.y - 1, 0);
          this.set(l.x, l.y - 2, 0);
        }
        else if (dir == Direction.WEST) 
        {
          stack.push(new Point(l.x - 2, l.y));
          this.set(l.x - 1, l.y, 0);
          this.set(l.x - 2, l.y, 0);
        }
        else if (dir == Direction.EAST) 
        {
          stack.push(new Point(l.x + 2, l.y));
          this.set(l.x + 1, l.y, 0);
          this.set(l.x + 2, l.y, 0);
        }
      }
    }
  }
  this.getPossibleDirections = function(p) 
  {
    var pos = new Array();
    if (p.x - 2 >= 0 && this.get(p.x - 2, p.y) == 1) {
      pos.push(Direction.WEST);
    }
    if (p.x + 2 < this.width && this.get(p.x + 2, p.y) == 1) {
      pos.push(Direction.EAST);
    }
    if (p.y - 2 >= 0 && this.get(p.x, p.y - 2) === 1) {
			pos.push(Direction.NORTH);
    }
    if (p.y + 2 < this.height && this.get(p.x, p.y + 2) == 1) {
      pos.push(Direction.SOUTH);
    }
    return pos;

  }
  this.getSomeDirection = function(p) 
  {
    dirs = this.getPossibleDirections(p);
    return dirs[Math.floor(Math.random() * dirs.length)];
  }
  
  this.generate();
}

